import re
import os
import sys
import cv

import feature
import knn
import preprocess

from scipy.io import loadmat

knn_k = 5
vocabulary = loadmat('vocabulary.mat')['vocabulary']
model = knn.load_model('model.mat')

def classify(input):
    im = cv.LoadImage(input)
    parts = preprocess.split(preprocess.denoise(im))
    vec = [feature.vectorize_sample(x, vocabulary) for x in parts]
    lbl = [knn.classify(x, model, knn_k) for x in vec]
    lbl = [chr(ord('A')+x) for x in lbl]
    return ''.join(lbl)

if __name__ == '__main__':
    to_test = sys.argv[1]
    if os.path.isdir(to_test):
        dict = {}
        pat = re.compile(r'^(?P<fn>[^:]+):.,(?P<lbl>[A-Z]{4}),.*')
        for l in open('%s/judgeData.txt' % to_test):
            md= pat.match(l)
            if md is not None:
                dict[md.group('fn')] = md.group('lbl')

        for f in sorted(os.listdir(to_test)):
            if not f.endswith('.bmp'):
                continue
            lbl = classify('%s/%s' % (to_test, f))
            if lbl == dict[f]:
                print '[T] %s: %s' % (f, lbl)
            else:
                print '[F] %s: %s (should be %s)' % (f, lbl, dict[f])
    else:
        print '%s: %s' % (to_test, classify(to_test))
