import cv
from numpy import zeros, array, tile, matlib
from scipy.cluster.vq import kmeans

def get_surf_descriptors(im):
    if im.nChannels > 1:
        im_gray = cv.CreateImage(cv.GetSize(im), cv.IPL_DEPTH_8U, 1)
        cv.CvtColor(im, im_gray, cv.CV_RGB2GRAY)
    else:
        im_gray = im
    kp, desc = cv.ExtractSURF(im_gray, None, cv.CreateMemStorage(),
                              (0,   # whether extended
                               200, # hessian threshold
                               5,   # number of octaves
                               4))  # number of layers
    return desc

def build_vocabulary(descriptors, vocabulary_size):
    nsmp = sum([len(x) for x in descriptors])
    nfea = len(descriptors[0][0])
    data = zeros((nsmp, nfea))

    print 'Building vocabulary...'
    print '  sample count:    %d' % nsmp
    print '  vocabulary size: %d' % vocabulary_size

    i = 0
    for smp in descriptors:
        for d in smp:
            data[i] = array(d)
            i += 1
    centroids, disto = kmeans(data, vocabulary_size)
    return centroids

def vectorize_sample(im, vocabulary):
    desc = get_surf_descriptors(im)
    return vectorize_descriptors(desc, vocabulary)

def vectorize_descriptors(desc, vocabulary):
    vec = zeros(vocabulary.shape[0])
    for d in desc:
        x = array(d)
        tmp = tile(x, (vocabulary.shape[0], 1))-vocabulary
        dist = matlib.sum(tmp*tmp, axis=1)
        idx = dist.argmin()
        vec[idx] += 1
    return vec


