import os
import cv
import sys
import numpy
import scipy.io

def scale_image(im):
    if im.nChannels > 1:
        im_gray = cv.CreateImage(cv.GetSize(im), cv.IPL_DEPTH_8U, 1)
        cv.CvtColor(im, im_gray, cv.CV_RGB2GRAY)
        im = im_gray
    threshold = 2
    x1 = 0
    while x1 < im.width:
        acc = 0
        for j in range(im.height):
            if im[j,x1] != 255: acc += 1
        x1 += 1
        if acc > threshold: break
    x2 = im.width-1
    while x2 >= 0:
        acc = 0
        for j in range(im.height):
            if im[j,x2] != 255: acc += 1
        x2 -= 1
        if acc > threshold: break
    y1 = 0
    while y1 < im.height:
        acc = 0
        for i in range(im.width):
            if im[y1, i] != 255: acc += 1
        y1 += 1
        if acc > threshold: break
    y2 = im.height-1
    while y2 >= 0:
        acc = 0
        for i in range(im.width):
            if im[y2, i] != 255: acc += 1
        y2 -= 1
        if acc > threshold: break
    region = cv.GetSubRect(im, (x1, y1, x2-x1+1, y2-y1+1))
    src = cv.CreateImage(cv.GetSize(region), cv.IPL_DEPTH_8U, 1)
    cv.Copy(region, src)
    dst = cv.CreateImage((50, 50), cv.IPL_DEPTH_8U, 1)
    cv.Resize(src, dst, cv.CV_INTER_CUBIC)
    return dst

def cv2array(im):
    depth2dtype = {
        cv.IPL_DEPTH_8U: 'uint8',
        cv.IPL_DEPTH_8S: 'int8',
        cv.IPL_DEPTH_16U: 'uint16',
        cv.IPL_DEPTH_16S: 'int16',
        cv.IPL_DEPTH_32S: 'int32',
        cv.IPL_DEPTH_32F: 'float32',
        cv.IPL_DEPTH_64F: 'float64',
    }

    arrdtype=im.depth
    a = numpy.fromstring(
        im.tostring(),
        dtype=depth2dtype[im.depth],
        count=im.width*im.height*im.nChannels)
    a.shape = (im.height,im.width,im.nChannels)
    return a

def cv2vec(im):
    if im.nChannels > 1:
        im_gray = cv.CreateImage(cv.GetSize(im), cv.IPL_DEPTH_8U, 1)
        cv.CvtColor(im, im_gray, cv.CV_RGB2GRAY)
        im = im_gray
    bin = cv.CreateImage(cv.GetSize(im), cv.IPL_DEPTH_8U, 1)
    cv.Threshold(im, bin, 254, 1, cv.CV_THRESH_BINARY_INV)
    a = cv2array(bin)
    a.shape = a.shape[0]*a.shape[1]
    return a

def load_coef(filename):
    return scipy.io.loadmat(filename)['eigvec']

def vectorize_sample(im, coef):
    return numpy.dot(cv2vec(scale_image(im)), coef)

if __name__ == '__main__':
    if len(sys.argv) < 2:
        print 'Usage:'
        print '    %s [scale|prepare] input output' % sys.argv[0]
        sys.exit(0)
    cmd = sys.argv[1]
    if cmd == 'scale':
        input = sys.argv[2]
        output = sys.argv[3]
        if not os.path.isdir(output): os.mkdir(output)
        for f in os.listdir(input):
            if not f.endswith('.png'): continue
            im = cv.LoadImage('%s/%s' % (input, f))
            cv.SaveImage('%s/%s' % (output, f), scale_image(im))
    elif cmd == 'prepare':
        # convert image into vector, prepare data for (matlab) PCA
        # should use the scaled version of image, we'll not scale again
        # here
        input = sys.argv[2]
        output = sys.argv[3]
        files = []
        for f in os.listdir(input):
            if not f.endswith('.png'): continue
            files.append(f)

        X = numpy.zeros((len(files), 50*50))
        for i in range(len(files)):
            im = cv.LoadImage('%s/%s' % (input, files[i]))
            vec = cv2vec(im)
            X[i] = vec
        scipy.io.savemat(output, {'X': X})

